@@ -1,4 +1,4 @@
-/*  Copyright 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by 
+/*  Copyright 2001-2005 by Hans Reiser, licensing governed by 
     reiser4progs/COPYING.
     
     librepair/disk_scan.c - Disk scan pass of reiser4 filesystem recovery. 
@@ -52,7 +52,7 @@ static void repair_disk_scan_update(repair_ds_t *ds) {
 static errno_t cb_count_sd(reiser4_place_t *place, void *data) {
 	repair_ds_t *ds = (repair_ds_t *)data;
 
-	if (place->plug->id.group == STAT_ITEM)
+	if (place->plug->p.id.group == STAT_ITEM)
 		ds->stat.tmp++;
 
 	return 0;
