@@ -1,4 +1,4 @@
-/* Copyright 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by 
+/* Copyright 2001-2005 by Hans Reiser, licensing governed by 
    reiser4progs/COPYING.
    
    librepair/node.c - methods are needed for node recovery. */
@@ -55,7 +55,7 @@ static errno_t repair_node_items_check(reiser4_node_t *node, place_func_t func,
 		
 		aal_memcpy(&key, &place.key, sizeof(key));
 
-		if ((ret = repair_key_check_struct(&key)) < 0)
+		if ((ret =  objcall(&key, check_struct) < 0))
 			return ret;
 		
 		if (ret) {
@@ -68,7 +68,7 @@ static errno_t repair_node_items_check(reiser4_node_t *node, place_func_t func,
 			
 			goto error_remove_item;
 		} else if (reiser4_key_compfull(&key, &place.key)) {
-			/* Key has been fixed. */
+			/* @Key has been fixed. */
 			fsck_mess("Node (%llu): The key [%s] of the item (%u) "
 				  "is broken. %s [%s].", node->block->nr,
 				  reiser4_print_key(&place.key),
@@ -92,8 +92,8 @@ static errno_t repair_node_items_check(reiser4_node_t *node, place_func_t func,
 		if (ret & RE_FATAL) {
 			fsck_mess("Node (%llu), item (%u), [%s]: broken item "
 				  "found.%s", node->block->nr, pos->item,
-				  reiser4_print_key(&place.key),
-				  mode == RM_BUILD ? " Remove it." : "");
+				  reiser4_print_key(&key), mode == RM_BUILD ? 
+				  " Remove it." : "");
 
 			goto error_remove_item;
 		}
@@ -110,10 +110,23 @@ static errno_t repair_node_items_check(reiser4_node_t *node, place_func_t func,
 				return RE_FATAL;
 			}
 		}
+		
+		prev = key;
 
 		if ((ret = reiser4_item_maxreal_key(&place, &key)))
 			return ret;
 		
+		/* Check that the maxreal key is not less than the place key */
+		if (reiser4_key_compfull(&prev, &key) > 0) {
+			fsck_mess("Node (%llu), item (%u): has the wrong key "
+				  "or too large body: from [%s] to [%s].",
+				  node->block->nr, pos->item,
+				  reiser4_print_key(&prev),
+				  reiser4_print_key(&key));
+			
+			goto error_remove_item;
+		}
+		
 		if (func && (ret = func(&place, data)))
 			return ret;
 		
@@ -184,7 +197,7 @@ errno_t repair_node_check_level(reiser4_node_t *node, uint8_t mode) {
 			fsck_mess("Node (%llu): Node level (%u) does not match "
 				  "to the item type (%s).", node->block->nr, 
 				  reiser4_node_get_level(node),
-				  place.plug->label);
+				  place.plug->p.label);
 			
 			return RE_FATAL;
 		}
@@ -202,7 +215,7 @@ errno_t repair_node_check_struct(reiser4_node_t *node, place_func_t func,
 	aal_assert("vpf-494", node != NULL);
 	aal_assert("vpf-220", node->plug != NULL);
 	
-	res = plug_call(node->plug->o.node_ops, check_struct, node, mode);
+	res = objcall(node, check_struct, mode);
 	
 	if (repair_error_fatal(res))
 		return res;
@@ -243,7 +256,7 @@ errno_t repair_node_pack(reiser4_node_t *node,
 	aal_assert("umka-2622", node != NULL);
 	aal_assert("umka-2623", stream != NULL);
 
-	return plug_call(node->plug->o.node_ops, pack, node, stream);
+	return objcall(node, pack, stream);
 }
 
 /* Create node from passed @stream. */
@@ -286,8 +299,8 @@ reiser4_node_t *repair_node_unpack(reiser4_tree_t *tree,
 	aal_block_fill(block, 0);
 	
 	/* Requesting the plugin for initialization node entity. */
-	if (!(node = plug_call(plug->o.node_ops, unpack, block, 
-			       tree->key.plug, stream)))
+	if (!(node = plugcall((reiser4_node_plug_t *)plug, unpack, 
+			      block, tree->key.plug, stream)))
 	{
 		goto error_free_block;
 	}
@@ -307,6 +320,5 @@ void repair_node_print(reiser4_node_t *node, aal_stream_t *stream) {
 	aal_assert("umka-1537", node != NULL);
 	aal_assert("umka-1538", stream != NULL);
 	
-	plug_call(node->plug->o.node_ops, print, 
-		  node, stream, -1, -1, 0);
+	plugcall(node->plug, print, node, stream, -1, -1, 0);
 }
