@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    tail40.c -- reiser4 tail (formatting) item plugin. */
@@ -16,9 +16,7 @@ uint32_t tail40_units(reiser4_place_t *place) {
 }
 
 /* Returns key from tail at @place. */
-errno_t tail40_fetch_key(reiser4_place_t *place, 
-			 reiser4_key_t *key) 
-{
+errno_t tail40_fetch_key(reiser4_place_t *place, reiser4_key_t *key) {
 	uint32_t pos;
 	
 	aal_assert("vpf-627", key != NULL);
@@ -29,13 +27,12 @@ errno_t tail40_fetch_key(reiser4_place_t *place,
 }
 
 /* Reads units from tail at @place into passed @hint. */
-int64_t tail40_read_units(reiser4_place_t *place,
-			  trans_hint_t *hint)
-{
+int64_t tail40_read_units(reiser4_place_t *place, trans_hint_t *hint) {
 	uint32_t count;
 	
 	aal_assert("umka-1674", hint != NULL);
 	aal_assert("umka-1673", place != NULL);
+	aal_assert("vpf-1859", hint->specific != NULL);
 
 	count = hint->count;
 
@@ -67,12 +64,15 @@ errno_t tail40_prep_write(reiser4_place_t *place, trans_hint_t *hint) {
 	aal_assert("umka-2437", place != NULL);
 	aal_assert("umka-3113", place->node != NULL);
 	
+	hint->overhead = place->off;
+	
 	/* Check if we want to create new tail item. If so, we say, that we need
 	   @hint->count bytes in tree. Even if this is more than one node can
 	   fit, it is okay, because write function will actually write only
 	   amount of data which may fit into node at passed @place. */
 	if (place->pos.unit == MAX_UINT32) {
 		hint->len = hint->count;
+
 		aal_memcpy(&hint->maxkey, &hint->offset, sizeof(hint->maxkey));
 	} else {
 		uint32_t right;
@@ -83,30 +83,22 @@ errno_t tail40_prep_write(reiser4_place_t *place, trans_hint_t *hint) {
 		/* Item already exists. We will rewrite some part of it and some
 		   part have to be append. */
 		right = place->len - tail40_pos(place);
+		hint->len = right ? 0 : hint->count;
 		
-		hint->len = (right >= hint->count ? 0 :
-			     hint->count - right);
-
 		/* Getting maximal real key. It will be needed to determine if
 		   we insert data inside tail or behind it. */
 		tail40_maxreal_key(place, &hint->maxkey);
 
-		max_offset = plug_call(hint->maxkey.plug->o.key_ops,
-				       get_offset, &hint->maxkey) + 1;
-
-		plug_call(hint->maxkey.plug->o.key_ops,
-			  set_offset, &hint->maxkey, max_offset);
+		max_offset = objcall(&hint->maxkey, get_offset) + 1;
+		objcall(&hint->maxkey, set_offset, max_offset);
 	}
 
 	/* Max possible item size. */
-	space = plug_call(place->node->plug->o.node_ops,
-			  maxspace, place->node);
+	space = objcall(place->node, maxspace);
 	
-	hint->overhead = place->off;
-
-	if (hint->len > space)
+	if (hint->len > (int32_t)(space - hint->overhead))
 		hint->len = space - hint->overhead;
-		
+	
 	return 0;
 }
 
@@ -131,12 +123,10 @@ int64_t tail40_write_units(reiser4_place_t *place, trans_hint_t *hint) {
 		count = place->len - tail40_pos(place);
 
 	/* Getting old max real offset. */
-	max_offset = plug_call(hint->maxkey.plug->o.key_ops,
-			       get_offset, &hint->maxkey);
+	max_offset = objcall(&hint->maxkey, get_offset);
 
 	/* Getting insert offset. */
-	ins_offset = plug_call(hint->offset.plug->o.key_ops,
-			       get_offset, &hint->offset);
+	ins_offset = objcall(&hint->offset, get_offset);
 	
 	/* Checking if we insert a hole. That is @hint->specific si null. If so,
 	   then we write @count of zeros. Writing data from @hint->specific
@@ -191,13 +181,9 @@ lookup_t tail40_lookup(reiser4_place_t *place,
 	aal_assert("umka-1229", hint != NULL);
 	aal_assert("umka-1228", place != NULL);
 
-	units = tail40_units(place);
-	
-	offset = plug_call(hint->key->plug->o.key_ops,
-			   get_offset, &place->key);
-
-	wanted = plug_call(hint->key->plug->o.key_ops,
-			   get_offset, hint->key);
+	units = tail40_units(place);	
+	offset = objcall(&place->key, get_offset);
+	wanted = objcall(hint->key, get_offset);
 
 	/* Check if needed key is inside this tail. */
 	if (wanted >= offset && wanted < offset + units) {
@@ -210,12 +196,15 @@ lookup_t tail40_lookup(reiser4_place_t *place,
 }
 
 #ifndef ENABLE_MINIMAL
+/* FIXME: *_place->off is not properly handled here. */
 /* Estimates how many bytes may be shifted from @stc_place to @dst_place. */
 errno_t tail40_prep_shift(reiser4_place_t *src_place,
 			  reiser4_place_t *dst_place,
 			  shift_hint_t *hint)
 {
 	int check_point;
+	uint32_t space;
+	uint32_t overhead;
 
 	aal_assert("umka-2279", hint != NULL);
 	aal_assert("umka-1664", src_place != NULL);
@@ -223,16 +212,28 @@ errno_t tail40_prep_shift(reiser4_place_t *src_place,
 	check_point = (src_place->pos.item == hint->pos.item &&
 		       hint->pos.unit != MAX_UINT32);
 
+	space = hint->units_bytes;
+	
+	/* If a new item is being created, substract the overhead. */
+	overhead = hint->create ? src_place->off : 0;
+	
+	if (space <= overhead) {
+		hint->units_bytes = hint->units_number = 0;
+		return 0;
+	}
+
+	space -= overhead;
+	
 	/* Check if this is left shift. */
 	if (hint->control & SF_ALLOW_LEFT) {
 		/* Check if should take into account inert point from @hint. */
 		if (hint->control & SF_UPDATE_POINT && check_point) {
 			/* Correcting @hint->rest. It should contain number of
 			   bytes we realy can shift. */
-			if (hint->units_bytes > hint->pos.unit)
-				hint->units_bytes = hint->pos.unit;
+			if (space > hint->pos.unit)
+				space = hint->pos.unit;
 
-			hint->pos.unit -= hint->units_bytes;
+			hint->pos.unit -= space;
 
 			/* Moving insert point to @dst_place. */
 			if (hint->pos.unit == 0 &&
@@ -240,14 +241,13 @@ errno_t tail40_prep_shift(reiser4_place_t *src_place,
 			{
 				hint->result |= SF_MOVE_POINT;
 
-				hint->pos.unit = hint->units_bytes +
+				hint->pos.unit = space +
 					(dst_place ? dst_place->len - 
 					 dst_place->off : 0);
 			}
 		} else {
-			if (hint->units_bytes + src_place->off > src_place->len)
-				hint->units_bytes = 
-					src_place->len - src_place->off;
+			if (space + src_place->off > src_place->len)
+				space = src_place->len - src_place->off;
 		}
 	} else {
 		uint32_t right;
@@ -261,13 +261,13 @@ errno_t tail40_prep_shift(reiser4_place_t *src_place,
 
 				/* Insert point inside item and we can move
 				   something. */
-				if (hint->units_bytes > right)
-					hint->units_bytes = right;
+				if (space > right)
+					space = right;
 
-				/* Updating insert point to first position in
-				   neighbour item. */
-				if (hint->pos.unit + src_place->off >= src_place->len && 
-				    hint->control & SF_MOVE_POINT)
+				/* If all @right units are shifted, update the 
+				   point if needed. */
+				if ((space == right) && 
+				    (hint->control & SF_MOVE_POINT))
 				{
 					hint->result |= SF_MOVE_POINT;
 					hint->pos.unit = 0;
@@ -280,19 +280,16 @@ errno_t tail40_prep_shift(reiser4_place_t *src_place,
 					hint->pos.unit = 0;
 				}
 
-				hint->units_bytes = 0;
+				space = 0;
 			}
 		} else {
-			if (hint->units_bytes + src_place->off > src_place->len)
-			{
-				hint->units_bytes = 
-					src_place->len - src_place->off;
-			}
+			if (space + src_place->off > src_place->len)
+				space = src_place->len - src_place->off;
 		}
 	}
 
-	hint->units_number = hint->units_bytes;
-	
+	hint->units_bytes = space + overhead;
+	hint->units_number = space;
 	return 0;
 }
 
@@ -368,6 +365,9 @@ errno_t tail40_shift_units(reiser4_place_t *src_place,
 	aal_assert("umka-1666", dst_place != NULL);
 	aal_assert("umka-1667", hint != NULL);
 
+	if (hint->create)
+		hint->units_bytes -= src_place->off;
+	
 	/* Check if this is left shift. */
 	if (hint->control & SF_ALLOW_LEFT) {
 		pos = dst_place->len - dst_place->off - hint->units_number;
@@ -388,12 +388,10 @@ errno_t tail40_shift_units(reiser4_place_t *src_place,
 
 		/* Updating @place->key in order to maintain consistency of left
 		   delimiting keys. */
-		offset = plug_call(src_place->key.plug->o.key_ops,
-				   get_offset, &src_place->key);
+		offset = objcall(&src_place->key, get_offset);
 
-		plug_call(src_place->key.plug->o.key_ops,
-			  set_offset, &src_place->key,
-			  offset + hint->units_bytes);
+		objcall(&src_place->key, set_offset, 
+			offset + hint->units_bytes);
 	} else {
 		/* Right shift. Expanding @dst_place at 0 pos. */
 		tail40_expand(dst_place, 0, hint->units_number);
@@ -408,12 +406,10 @@ errno_t tail40_shift_units(reiser4_place_t *src_place,
 			      hint->units_bytes);
 
 		/* Updating place key. */
-		offset = plug_call(dst_place->key.plug->o.key_ops,
-				   get_offset, &dst_place->key);
+		offset = objcall(&dst_place->key, get_offset);
 		
-		plug_call(dst_place->key.plug->o.key_ops,
-			  set_offset, &dst_place->key,
-			  offset - hint->units_bytes);
+		objcall(&dst_place->key, set_offset, 
+			offset - hint->units_bytes);
 	}
 	
 	return 0;
@@ -451,7 +447,7 @@ int64_t tail40_trunc_units(reiser4_place_t *place, trans_hint_t *hint) {
 			       &place->key, NULL);
 	}
 	
-	hint->overhead = count == (place->len - place->off) ? place->off : 0;
+	hint->overhead = (count == place->len - place->off) ? place->off : 0;
 	hint->len = count;
 	hint->bytes = count;
 	
