@@ -1,4 +1,4 @@
-/* Copyright 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    node40_repair.c -- reiser4 node with short keys. */
@@ -23,9 +23,8 @@ static void node40_set_offset_at(reiser4_node_t *node, int pos,
 	}
 }
 
-static errno_t node40_region_delete(reiser4_node_t *node,
-				    uint16_t start_pos, 
-				    uint16_t end_pos) 
+errno_t node40_region_delete(reiser4_node_t *node,
+			     uint16_t start_pos, uint16_t end_pos)
 {
 	uint32_t count;
 	uint32_t len;
@@ -59,7 +58,7 @@ static errno_t node40_region_delete(reiser4_node_t *node,
 
 /* Count is valid if:
    free_space_start + free_space == block_size - count * ih size */
-static int node40_count_valid(reiser4_node_t *node) {
+int node40_count_valid(reiser4_node_t *node) {
 	uint32_t count;
 	
 	count = nh_get_num_items(node);
@@ -77,15 +76,13 @@ static int node40_count_valid(reiser4_node_t *node) {
 		count * ih_size(node->keypol) == node->block->size);
 }
 
-/* Look through ih array looking for the last valid item location. This will 
-   be the last valid item. */
-static uint32_t node40_estimate_count(reiser4_node_t *node) {
+uint32_t node40_estimate_count_common(reiser4_node_t *node,
+				      uint32_t node_header_size)
+{
 	uint32_t offset, left, right;
-	uint32_t count, last, i;
-	
-	count = nh_get_num_items(node);
-	
-	left = sizeof(node40_header_t);
+	uint32_t last, i;
+
+	left = node_header_size;
 	right = node->block->size - ih_size(node->keypol) - MIN_ITEM_LEN;
 	last = 0;
 
@@ -106,8 +103,14 @@ static uint32_t node40_estimate_count(reiser4_node_t *node) {
 	return last + 1;
 }
 
-static errno_t node40_space_check(reiser4_node_t *node, 
-				  uint32_t offset, uint8_t mode)
+/* Look through ih array looking for the last valid item location. This will
+   be the last valid item. */
+static uint32_t node40_estimate_count(reiser4_node_t *node)
+{
+	return node40_estimate_count_common(node, sizeof(node40_header_t));
+}
+
+errno_t node40_space_check(reiser4_node_t *node, uint32_t offset, uint8_t mode)
 {
 	errno_t res = 0;
 	uint32_t space;
@@ -153,9 +156,9 @@ static errno_t node40_space_check(reiser4_node_t *node,
 	return res;
 }
 
-/* Count of items is correct. Free space fields and item locations should be 
- * checked/recovered if broken. */
-static errno_t node40_ih_array_check(reiser4_node_t *node, uint8_t mode) {
+errno_t node40_ih_array_check_common(reiser4_node_t *node, uint8_t mode,
+				     uint32_t node_header_size)
+{
 	uint32_t right, left, offset;
 	uint32_t last_pos, count, i;
 	errno_t res = 0;
@@ -170,8 +173,8 @@ static errno_t node40_ih_array_check(reiser4_node_t *node, uint8_t mode) {
 	offset = 0;
 	last_pos = 0;
 	count = nh_get_num_items(node);
-	
-	left = sizeof(node40_header_t);
+
+	left = node_header_size;
 	right = node->block->size - count * ih_size(node->keypol);
 	
 	for(i = 0; i <= count; i++, left += MIN_ITEM_LEN) {
@@ -253,13 +256,23 @@ static errno_t node40_ih_array_check(reiser4_node_t *node, uint8_t mode) {
 	return res;
 }
 
-/* Checks the count of items written in node_header. If it is wrong, it tries
-   to estimate it on the base of free_space fields and recover if REBUILD mode.
-   Returns FATAL otherwise. */
-static errno_t node40_count_check(reiser4_node_t *node, uint8_t mode) {
+/*
+ * Count of items is correct. Free space fields and item locations should be
+ * checked/recovered if broken
+ */
+errno_t node40_ih_array_check(reiser4_node_t *node, uint8_t mode)
+{
+	return node40_ih_array_check_common(node, mode,
+					    sizeof(node40_header_t));
+}
+
+errno_t node40_count_check_common(reiser4_node_t *node,
+				  uint8_t mode,
+				  uint32_t (*estimate_count)(reiser4_node_t *))
+{
 	uint32_t num, count;
 	blk_t blk;
-	
+
 	aal_assert("vpf-802", node != NULL);
 	aal_assert("vpf-803", node->block != NULL);
 	
@@ -270,7 +283,7 @@ static errno_t node40_count_check(reiser4_node_t *node, uint8_t mode) {
 		return 0;
 	
 	/* Count is probably is not valid. Estimate the count. */
-	num = node40_estimate_count(node);
+	num = estimate_count(node);
 	count = nh_get_num_items(node);
 	
 	if (num >= count)
@@ -290,7 +303,15 @@ static errno_t node40_count_check(reiser4_node_t *node, uint8_t mode) {
 	return 0;
 }
 
-static errno_t node40_iplug_check(reiser4_node_t *node, uint8_t mode) {
+/* Checks the count of items written in node_header. If it is wrong, it tries
+   to estimate it on the base of free_space fields and recover if REBUILD mode.
+   Returns FATAL otherwise. */
+static errno_t node40_count_check(reiser4_node_t *node, uint8_t mode)
+{
+	return node40_count_check_common(node, mode, node40_estimate_count);
+}
+
+errno_t node40_iplug_check(reiser4_node_t *node, uint8_t mode) {
 	uint32_t count, len;
 	uint16_t pid;
 	errno_t res;
@@ -374,28 +395,16 @@ int64_t node40_insert_raw(reiser4_node_t *entity, pos_t *pos,
 	aal_assert("vpf-1368", hint != NULL);
 	
 	return node40_modify(entity, pos, hint, 
-			     hint->plug->o.item_ops->repair->insert_raw);
+			     hint->plug->repair->insert_raw);
 }
 
-errno_t node40_pack(reiser4_node_t *entity, aal_stream_t *stream) {
+/*
+ * Pack node40 header w/out magic and padding
+ */
+void node40_header_pack(reiser4_node_t *entity, aal_stream_t *stream)
+{
 	node40_header_t *head;
-	reiser4_place_t place;
-	uint16_t num;
-	pos_t *pos;
-	rid_t pid;
-	
-	aal_assert("umka-2596", entity != NULL);
-	aal_assert("umka-2598", stream != NULL);
-	
-	pid = entity->plug->id.id;
-	aal_stream_write(stream, &pid, sizeof(pid));
-	
-	/* Write node block number. */
-	aal_stream_write(stream, &entity->block->nr,
-			 sizeof(entity->block->nr));
 
-	/* Pack the node content. */
-	
 	/* Node header w/out magic and padding. */
 	head = nh(entity->block);
 	
@@ -419,51 +428,208 @@ errno_t node40_pack(reiser4_node_t *entity, aal_stream_t *stream) {
 
 	aal_stream_write(stream, &head->level, 
 			 sizeof(head->level));
+}
+
+int32_t node40_items_pack(reiser4_node_t *entity, aal_stream_t *stream)
+{
+	reiser4_place_t place;
+	uint16_t num;
+	pos_t *pos;
 
-	/* All items. */
 	num = nh_get_num_items(entity);
 
 	pos = &place.pos;
 	pos->unit = MAX_UINT32;
-	
-	/* Pack all item headers. */
+	/*
+	 * Pack all item headers
+	 */
 	for (pos->item = 0; pos->item < num; pos->item++) {
 		void *ih = node40_ih_at(entity, pos->item);
 		aal_stream_write(stream, ih, ih_size(entity->keypol));
 	}
-
-	/* Pack all item bodies. */
+	/*
+	 * Pack all item bodies
+	 */
 	for (pos->item = 0; pos->item < num; pos->item++) {
 		if (node40_fetch(entity, pos, &place))
 			return -EINVAL;
-		
-		if (place.plug->o.item_ops->repair->pack) {
-			/* Pack body. */
-			if (plug_call(place.plug->o.item_ops->repair,
-				      pack, &place, stream))
-			{
+
+		if (place.plug->repair->pack) {
+			/*
+			 * Pack body
+			 */
+			if (objcall(&place, repair->pack, stream))
 				return -EINVAL;
-			}
 		} else {
-			/* Do not pack body. */
+			/*
+			 * Do not pack body
+			 */
 			aal_stream_write(stream, node40_ib_at(entity, pos->item),
 					 node40_len(entity, &place.pos));
 		}
 	}
-	
 	return 0;
 }
 
-reiser4_node_t *node40_unpack(aal_block_t *block,
-			     reiser4_plug_t *kplug,
-			     aal_stream_t *stream)
+errno_t node40_pack_common(reiser4_node_t *entity, aal_stream_t *stream,
+			   void (*pack_header_fn)(reiser4_node_t *,
+					     aal_stream_t *),
+			   int32_t (*pack_items_fn)(reiser4_node_t *,
+						    aal_stream_t *))
+{
+	int32_t ret;
+	rid_t pid;
+
+	aal_assert("umka-2596", entity != NULL);
+	aal_assert("umka-2598", stream != NULL);
+
+	pid = entity->plug->p.id.id;
+	aal_stream_write(stream, &pid, sizeof(pid));
+	/*
+	 * Write node block number
+	 */
+	aal_stream_write(stream, &entity->block->nr,
+			 sizeof(entity->block->nr));
+	/*
+	 * Pack node header
+	 */
+	pack_header_fn(entity, stream);
+	/*
+	 * Pack items
+	 */
+	ret = pack_items_fn(entity, stream);
+	if (ret)
+		return -EINVAL;
+	return 0;
+}
+
+errno_t node40_pack(reiser4_node_t *entity, aal_stream_t *stream)
+{
+	return node40_pack_common(entity, stream,
+				  node40_header_pack, node40_items_pack);
+}
+
+/*
+ * Unpack node40 header w/out magic and padding
+ */
+int32_t node40_header_unpack(reiser4_node_t *entity, aal_stream_t *stream)
 {
-	node40_header_t *head;
-	reiser4_node_t *entity;
-	reiser4_place_t place;
 	uint32_t read;
-	uint16_t num;
+	node40_header_t *head;
+
+	head = nh(entity->block);
+
+	read = aal_stream_read(stream, &head->num_items, 
+			       sizeof(head->num_items));
+	
+	if (read != sizeof(head->num_items))
+		goto error;
+	
+	read = aal_stream_read(stream, &head->free_space, 
+			       sizeof(head->free_space));
+	
+	if (read != sizeof(head->free_space))
+		goto error;
+	
+	read = aal_stream_read(stream, &head->free_space_start, 
+			       sizeof(head->free_space_start));
+	
+	if (read != sizeof(head->free_space_start))
+		goto error;
+	
+	read = aal_stream_read(stream, &head->mkfs_id, 
+			       sizeof(head->mkfs_id));
+	
+	if (read != sizeof(head->mkfs_id))
+		goto error;
+	
+	read = aal_stream_read(stream, &head->flush_id, 
+			       sizeof(head->flush_id));
+	
+	if (read != sizeof(head->flush_id))
+		goto error;
+	
+	read = aal_stream_read(stream, &head->flags, 
+			       sizeof(head->flags));
+	
+	if (read != sizeof(head->flags))
+		goto error;
+	
+	read = aal_stream_read(stream, &head->level, 
+			       sizeof(head->level));
+
+	if (read != sizeof(head->level))
+		goto error;
+	return 0;
+ error:
+	return -1;
+}
+
+/*
+ * Unpack item headers and item bodies
+ */
+int32_t node40_items_unpack(reiser4_node_t *entity, aal_stream_t *stream)
+{
 	pos_t *pos;
+	uint16_t num;
+	uint32_t read;
+	reiser4_place_t place;
+
+	num = nh_get_num_items(entity);
+	pos = &place.pos;
+	pos->unit = MAX_UINT32;
+	/*
+	 * Unpack all item headers
+	 */
+	for (pos->item = 0; pos->item < num; pos->item++) {
+		void *ih = node40_ih_at(entity, pos->item);
+		read = aal_stream_read(stream, ih, ih_size(entity->keypol));
+
+		if (read != ih_size(entity->keypol))
+			goto error;
+	}
+	/*
+	 * Unpack all item bodies
+	 */
+	for (pos->item = 0; pos->item < num; pos->item++) {
+		if (node40_fetch(entity, pos, &place))
+			goto error;
+		if (place.plug->repair->unpack) {
+			/*
+			 * Unpack body
+			 */
+			if (objcall(&place, repair->unpack, stream))
+				goto error;
+		} else {
+			void *ib = node40_ib_at(entity, pos->item);
+			uint32_t len = node40_len(entity, &place.pos);
+			/*
+			 * Do not unpack body
+			 */
+			if (aal_stream_read(stream, ib, len) != (int32_t)len)
+				goto error;
+		}
+	}
+	return 0;
+ error:
+	return -1;
+}
+
+reiser4_node_t *
+node40_unpack_common(aal_block_t *block,
+		     reiser4_key_plug_t *kplug,
+		     aal_stream_t *stream,
+		     reiser4_node_plug_t *nplug,
+		     const uint32_t magic,
+		     reiser4_node_t* (*prepare_fn)(aal_block_t *,
+						  reiser4_key_plug_t *),
+		     int32_t (*unpack_header_fn)(reiser4_node_t *,
+						 aal_stream_t *),
+		     int32_t (*unpack_items_fn)(reiser4_node_t *,
+						aal_stream_t *))
+{
+	int32_t ret;
+	reiser4_node_t *entity;
 
 	aal_assert("umka-2597", block != NULL);
 	aal_assert("umka-2632", kplug != NULL);
@@ -472,119 +638,68 @@ reiser4_node_t *node40_unpack(aal_block_t *block,
 	if (!(entity = aal_calloc(sizeof(*entity), 0)))
 		return NULL;
 
-	entity->kplug = kplug;
-	entity->block = block;
-	entity->plug = &node40_plug;
-	
+	if (!(entity = prepare_fn(block, kplug))) {
+		aal_free(entity);
+		return NULL;
+	}
 	node40_mkdirty(entity);
-	
-	/* Unpack the node content. */
-	
-	/* Node header w/out magic and padding. */
-	head = nh(entity->block);
-
-	read = aal_stream_read(stream, &head->num_items, 
-			       sizeof(head->num_items));
-	
-	if (read != sizeof(head->num_items))
-		goto error_free_entity;
-	
-	read = aal_stream_read(stream, &head->free_space, 
-			       sizeof(head->free_space));
-	
-	if (read != sizeof(head->free_space))
-		goto error_free_entity;
-	
-	read = aal_stream_read(stream, &head->free_space_start, 
-			       sizeof(head->free_space_start));
-	
-	if (read != sizeof(head->free_space_start))
-		goto error_free_entity;
-	
-	read = aal_stream_read(stream, &head->mkfs_id, 
-			       sizeof(head->mkfs_id));
-	
-	if (read != sizeof(head->mkfs_id))
-		goto error_free_entity;
-	
-	read = aal_stream_read(stream, &head->flush_id, 
-			       sizeof(head->flush_id));
-	
-	if (read != sizeof(head->flush_id))
-		goto error_free_entity;
-	
-	read = aal_stream_read(stream, &head->flags, 
-			       sizeof(head->flags));
-	
-	if (read != sizeof(head->flags))
-		goto error_free_entity;
-	
-	read = aal_stream_read(stream, &head->level, 
-			       sizeof(head->level));
-	
-	if (read != sizeof(head->level))
-		goto error_free_entity;
-	
-	/* Set the magic and the pid. */
-	nh_set_magic(entity, NODE40_MAGIC);
-	nh_set_pid(entity, node40_plug.id.id);
-	
-	/* All items. */
-	num = nh_get_num_items(entity);
-	pos = &place.pos;
-	pos->unit = MAX_UINT32;
-	
-	/* Unpack all item headers. */
-	for (pos->item = 0; pos->item < num; pos->item++) {
-		void *ih = node40_ih_at(entity, pos->item);
-		read = aal_stream_read(stream, ih, ih_size(entity->keypol));
-		
-		if (read != ih_size(entity->keypol))
-			goto error_free_entity;
-	}
-
-	/* Unpack all item bodies. */
-	for (pos->item = 0; pos->item < num; pos->item++) {
-		if (node40_fetch(entity, pos, &place))
-			goto error_free;
-		
-		if (place.plug->o.item_ops->repair->unpack) {
-			/* Unpack body. */
-			if (plug_call(place.plug->o.item_ops->repair,
-				      unpack, &place, stream))
-			{
-				goto error_free_entity;
-			}
-		} else {
-			void *ib = node40_ib_at(entity, pos->item);
-			uint32_t len = node40_len(entity, &place.pos);
-			
-			/* Do not unpack body. */
-			if (aal_stream_read(stream, ib, len) != (int32_t)len)
-				goto error_free_entity;
-		}
-	}
-	
+	/*
+	 * Unpack the node header content
+	 */
+	ret = unpack_header_fn(entity, stream);
+	if (ret)
+		goto error;
+	/*
+	 * Set the magic and the pid
+	 */
+	nh_set_magic(entity, magic);
+	nh_set_pid(entity, nplug->p.id.id);
+	/*
+	 * Unpack item headers and item bodies
+	 */
+	ret = unpack_items_fn(entity, stream);
+	if (ret)
+		goto error;
 	return entity;
-	
- error_free_entity:
+ error:
 	aal_error("Can't unpack the node (%llu). "
 		  "Stream is over?", block->nr);
-
- error_free:
 	aal_free(entity);
 	return NULL;
 }
 
-/* Prepare text node description and push it into specified @stream. */
-void node40_print(reiser4_node_t *entity, aal_stream_t *stream,
-		  uint32_t start, uint32_t count, uint16_t options) 
+reiser4_node_t *node40_unpack(aal_block_t *block,
+			     reiser4_key_plug_t *kplug,
+			     aal_stream_t *stream)
+{
+	return node40_unpack_common(block, kplug, stream,
+				    &node40_plug, NODE40_MAGIC,
+				    node40_prepare,
+				    node40_header_unpack,
+				    node40_items_unpack);
+}
+
+static void node40_header_print(reiser4_node_t *entity, aal_stream_t *stream)
+{
+	uint8_t level;
+	level = node40_get_level(entity);
+
+	aal_stream_format(stream, "NODE (%llu) LEVEL=%u ITEMS=%u "
+			  "SPACE=%u MKFS ID=0x%x FLUSH=0x%llx\n",
+			  entity->block->nr, level, node40_items(entity),
+			  node40_space(entity), nh_get_mkfs_id(entity),
+			  nh_get_flush_id(entity));
+}
+
+void node40_print_common(reiser4_node_t *entity, aal_stream_t *stream,
+			 uint32_t start, uint32_t count, uint16_t options,
+			 void (*print_header_fn)(reiser4_node_t *,
+						 aal_stream_t *))
 {
 	void *ih;
 	char *key;
 	pos_t pos;
 	uint8_t pol;
-	uint8_t level;
 
 	uint32_t last, num;
 	reiser4_place_t place;
@@ -592,15 +707,8 @@ void node40_print(reiser4_node_t *entity, aal_stream_t *stream,
 	aal_assert("vpf-023", entity != NULL);
 	aal_assert("umka-457", stream != NULL);
 
-	level = node40_get_level(entity);
-	
-	/* Print node header. */
-	aal_stream_format(stream, "NODE (%llu) LEVEL=%u ITEMS=%u "
-			  "SPACE=%u MKFS ID=0x%x FLUSH=0x%llx\n",
-			  entity->block->nr, level, node40_items(entity),
-			  node40_space(entity), nh_get_mkfs_id(entity),
-			  nh_get_flush_id(entity));
-	
+	print_header_fn(entity, stream);
+
 	pos.unit = MAX_UINT32;
 	
 	if (start == MAX_UINT32)
@@ -636,22 +744,23 @@ void node40_print(reiser4_node_t *entity, aal_stream_t *stream,
 		key = print_key(node40_core, &place.key);
 		
 		aal_stream_format(stream, "#%u%s %s (%s): [%s] OFF=%u, "
-				  "LEN=%u, flags=0x%x \n", pos.item,
+				  "LEN=%u, flags=0x%x", pos.item,
 				  pos.item >= num ? "D" : " ", place.plug ? 
-				  reiser4_igname[place.plug->id.group] : 
-				  "UNKN", place.plug ? place.plug->label :
+				  reiser4_igname[place.plug->p.id.group] : 
+				  "UNKN", place.plug ? place.plug->p.label :
 				  "UNKN", key, ih_get_offset(ih, pol),
 				  place.len, ih_get_flags(ih, pol));
 
 		/* Printing item by means of calling item print method if it is
 		   implemented. If it is not, then print common item information
 		   like key, len, etc. */
-		if (place.plug && place.plug->o.item_ops->debug->print &&
+		if (place.plug && place.plug->debug->print &&
 		    place.body - entity->block->data + place.len < 
 		    entity->block->size) 
 		{
-			plug_call(place.plug->o.item_ops->debug,
-				  print, &place, stream, options);
+			objcall(&place, debug->print, stream, options);
+		} else {
+			aal_stream_format(stream, "\n");
 		}
 	}
 	
@@ -659,145 +768,26 @@ void node40_print(reiser4_node_t *entity, aal_stream_t *stream,
 			  "===================================="
 			  "==============\n");
 }
-#if 0
-static bool_t node40_item_count_valid(reiser4_node_t *node,
-				      uint32_t count)
+
+/*
+ * Prepare text node description and push it into specified @stream
+ */
+void node40_print(reiser4_node_t *entity, aal_stream_t *stream,
+		  uint32_t start, uint32_t count, uint16_t options)
 {
-	uint32_t pol = node40_key_pol(node);
-	uint32_t blksize = node->block->size;
-	
-	return !( ( blksize - sizeof(node40_header_t) ) / 
-		  ( ih_size(pol) + MIN_ITEM_LEN ) 
-		  < count);
+	return node40_print_common(entity, stream, start, count, options,
+				   node40_header_print);
 }
 
-static uint32_t node40_get_count(reiser4_node_t *node) {
-	uint32_t num, blk_size, pol;
-	
-	aal_assert("vpf-804", node != NULL);
-	aal_assert("vpf-806", node->block != NULL);
-	
-	pol = node40_key_pol(node);
-	blk_size = node->block->size;
-	
-	/* Free space start is less then node_header + MIN_ITEM_LEN. */
-	if (sizeof(node40_header_t) + MIN_ITEM_LEN > 
-	    nh_get_free_space_start(node))
-		return 0;
-	
-	/* Free space start is greater then the first item  */
-	if (blk_size - ih_size(pol) < nh_get_free_space_start(node))
-		return 0;
-	
-	/* Free space + node_h + 1 item_h + 1 MIN_ITEM_LEN should be less them 
-	 * blksize. */
-	if (nh_get_free_space(node) > blk_size -
-	    sizeof(node40_header_t) - ih_size(pol) - MIN_ITEM_LEN)
-	{
-		return 0;
-	}
-	
-	num = nh_get_free_space_start(node) + nh_get_free_space(node);
-	
-	/* Free space end > first item header offset. */
-	if (num > blk_size - ih_size(pol))
-		return 0;
-	
-	num = blk_size - num;
-	
-	/* The space between free space end and the end of block should be 
-	 * divisible by item_header. */
-	if (num % ih_size(pol))
-		return 0;
-	
-	num /= ih_size(pol);
-	
-	if (!node40_item_count_valid(node, num))
-		return 0;
-	
-	return num;
-}
-
-static errno_t node40_item_find_array(reiser4_node_t *node, uint8_t mode) {
-	uint32_t offset, i, nr = 0;
-	uint32_t pol;
-	blk_t blk;
-	
-	aal_assert("vpf-800", node != NULL);
-	
-	blk = node->block->nr;
-	pol = node40_key_pol(node);
-	
-	for (i = 0; ; i++) {
-		offset = ih_get_offset(node40_ih_at(node, i), pol);
-		
-		if (i) {
-			if (offset < sizeof(node40_header_t) + i * MIN_ITEM_LEN)
-				break;	
-		} else {
-			if (offset != sizeof(node40_header_t))
-				return RE_FATAL;
-		}
-		
-		if (node->block->size - ih_size(pol) * (i + 1) -
-		    MIN_ITEM_LEN < offset)
-		{
-			break;
-		}
-		
-		nr++;
-	}
-	
-	/* Only nr - 1 item can be recovered as free space start is unknown. */
-	if (nr <= 1)
-		return RE_FATAL;
-	
-	if (--nr != nh_get_num_items(node)) {
-		fsck_mess("Node (%llu): Count of items (%u) is wrong. "
-			  "Found only (%u) items. %s", blk, 
-			  nh_get_num_items(node), nr, 
-			  mode == RM_BUILD ? "Fixed." : "");
-		
-		if (mode == RM_BUILD) {
-			nh_set_num_items(node, nr);
-			node40_mkdirty((reiser4_node_t *)node);
-		} else
-			return RE_FATAL;
-	}
-	
-	offset = ih_get_offset(node40_ih_at(node, nr + 1), pol);
-	if (offset != nh_get_free_space_start(node)) {
-		fsck_mess("Node (%llu): Free space start (%u) is wrong. "
-			  "(%u) looks correct. %s", blk, 
-			  nh_get_free_space_start(node), offset, 
-			  mode == RM_CHECK ? "" : "Fixed.");
-		
-		if (mode != RM_CHECK) {
-			nh_set_free_space_start(node, offset);
-			node40_mkdirty((reiser4_node_t *)node);
-		} else
-			return RE_FIXABLE;
-	}
-	
-	offset = node->block->size - offset - 
-		nr * ih_size(pol);
-	
-	if (offset != nh_get_free_space_start(node)) {
-		fsck_mess("Node (%llu): Free space (%u) is wrong. "
-			  "Should be (%u). %s", blk, 
-			  nh_get_free_space(node), offset, 
-			  mode == RM_CHECK ? "" : "Fixed.");
-		
-		if (mode != RM_CHECK) {
-			nh_set_free_space(node, offset);
-			node40_mkdirty((reiser4_node_t *)node);
-		} else
-			return RE_FIXABLE;
-	}
-	
-	return 0;
-}
 #endif
 
-
-#endif
+/*
+  Local variables:
+  c-indentation-style: "K&R"
+  mode-name: "LC"
+  c-basic-offset: 8
+  tab-width: 8
+  fill-column: 80
+  scroll-step: 1
+  End:
+*/
