@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
    
    sdext_symlink.c -- symlink stat data extension plugin. */
@@ -16,7 +16,7 @@ errno_t sdext_symlink_check_struct(stat_entity_t *stat, repair_hint_t *hint) {
 	uint32_t offset;
 	
 	aal_assert("vpf-779", stat != NULL);
-	aal_assert("vpf-780", stat->ext_plug != NULL);
+	aal_assert("vpf-780", stat->plug != NULL);
 	
 	offset = stat->offset;
 	
@@ -29,7 +29,7 @@ errno_t sdext_symlink_check_struct(stat_entity_t *stat, repair_hint_t *hint) {
 			  "like a valid (%s) statdata extension.", 
 			  place_blknr(stat->place), stat->place->pos.item,
 			  print_key(sdext_symlink_core, &stat->place->key),
-			  stat->ext_plug->label);
+			  stat->plug->p.label);
 
 		return RE_FATAL;
 	}
