@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2004 by Hans Reiser, licensing governed by
+/* Copyright (C) 2001-2005 by Hans Reiser, licensing governed by
    reiser4progs/COPYING.
 
    measurefs.c -- program for measuring reiser4. */
@@ -19,7 +19,7 @@
 #include <reiser4/libreiser4.h>
 
 /* Known measurefs behavior flags. */
-enum behav_flags {
+typedef enum behav_flags {
 	BF_FORCE      = 1 << 0,
 	BF_YES        = 1 << 1,
 	BF_TREE_FRAG  = 1 << 2,
@@ -29,9 +29,7 @@ enum behav_flags {
 	BF_SHOW_FILE  = 1 << 6,
 	BF_SHOW_PLUG  = 1 << 7,
 	BF_SHOW_PARM  = 1 << 8
-};
-
-typedef enum behav_flags behav_flags_t;
+} behav_flags_t;
 
 /* Prints measurefs options */
 static void measurefs_print_usage(char *name) {
@@ -72,15 +70,13 @@ static void measurefs_init(void) {
 		misc_exception_set_stream(ex, stderr);
 }
 
-struct tree_frag_hint {
+typedef struct tree_frag_hint {
 	aal_gauge_t *gauge;
 
 	blk_t curr;
 	uint16_t level;
 	count_t total, bad;
-};
-
-typedef struct tree_frag_hint tree_frag_hint_t;
+} tree_frag_hint_t;
 
 /* Open node callback for calculating the tree fragmentation */
 static reiser4_node_t *tree_frag_open_node(reiser4_tree_t *tree,
@@ -160,11 +156,10 @@ static errno_t tree_frag_process_node(reiser4_node_t *node, void *data) {
 		/* Checking and calling item's layout method with function
 		   tfrag_process_item() as a function for handling one block the
 		   item points to. */
-		if (!place.plug->o.item_ops->object->layout)
+		if (!place.plug->object->layout)
 			continue;
 
-		plug_call(place.plug->o.item_ops->object, layout,
-			  &place, tree_frag_process_item, data);
+		objcall(&place, object->layout, tree_frag_process_item, data);
 	}
 	
 	frag_hint->level--;
@@ -230,7 +225,7 @@ errno_t measurefs_tree_frag(reiser4_fs_t *fs, uint32_t flags) {
 	return 0;
 };
 
-struct tree_stat_hint {
+typedef struct tree_stat_hint {
 	aal_gauge_t *gauge;
 	reiser4_tree_t *tree;
 
@@ -253,9 +248,7 @@ struct tree_stat_hint {
 	double formatted_used;
 
 	reiser4_place_t *place;
-};
-
-typedef struct tree_stat_hint tree_stat_hint_t;
+} tree_stat_hint_t;
 
 /* Process one item on level > LEAF_LEVEL. */
 static errno_t stat_item_layout(uint64_t start, uint64_t width, void *data) {
@@ -266,7 +259,7 @@ static errno_t stat_item_layout(uint64_t start, uint64_t width, void *data) {
 	stat_hint = (tree_stat_hint_t *)data;
 	place = stat_hint->place;
 
-	if (place->plug->id.group == EXTENT_ITEM) {
+	if (place->plug->p.id.group == EXTENT_ITEM) {
 		reiser4_master_t *master;
 			
 		master = stat_hint->tree->fs->master;
@@ -360,7 +353,7 @@ static errno_t stat_process_node(reiser4_node_t *node, void *data) {
 		   item type independent manner. */
 		stat_hint->items++;
 
-		switch (place.plug->id.group) {
+		switch (place.plug->p.id.group) {
 		case STAT_ITEM:
 			stat_hint->statdatas++;
 			break;
@@ -380,12 +373,11 @@ static errno_t stat_process_node(reiser4_node_t *node, void *data) {
 
 		/* Calling layout() method with callback for counting referenced
 		   blocks. */
-		if (!place.plug->o.item_ops->object->layout)
+		if (!place.plug->object->layout)
 			continue;
 
 		stat_hint->place = &place;
-		plug_call(place.plug->o.item_ops->object, layout,
-			  &place, stat_item_layout, data);
+		objcall(&place, object->layout, stat_item_layout, data);
 	}
 
 	/* Updating common counters like nodes traversed at all, formatted ones,
@@ -478,7 +470,7 @@ errno_t measurefs_tree_stat(reiser4_fs_t *fs, uint32_t flags) {
 	return 0;
 }
 
-struct file_frag_hint {
+typedef struct file_frag_hint {
 	aal_gauge_t *gauge;
 
 	count_t bad;
@@ -490,9 +482,7 @@ struct file_frag_hint {
 	double current;
 	uint32_t flags;
 	uint16_t level;
-};
-
-typedef struct file_frag_hint file_frag_hint_t;
+} file_frag_hint_t;
 
 /* Callback function for processing one block belong to the file we are
    traversing. */
@@ -602,7 +592,7 @@ static errno_t data_frag_process_node(reiser4_node_t *node, void *data) {
 		   the function we have seen above. */
 		if (reiser4_object_layout(object, file_frag_process_blk, data)) {
 			aal_error("Can't enumerate data blocks occupied by %s", 
-				  reiser4_print_inode(&object->ent->object));
+				  reiser4_print_inode(&object->info.object));
 			goto error_close_object;
 		}
 
@@ -623,7 +613,7 @@ static errno_t data_frag_process_node(reiser4_node_t *node, void *data) {
 				(double)frag_hint->current / frag_hint->files : 0;
 			
 			aal_mess("Fragmentation for %s: %.6f [av. %.6f ]",
-				 reiser4_print_inode(&object->ent->object),
+				 reiser4_print_inode(&object->info.object),
 				 file_factor, curr_factor);
 		}
 
