@@ -207,7 +207,7 @@ int main(int argc, char *argv[]) {
 			}
 #if defined(HAVE_LIBUUID) && defined(HAVE_UUID_UUID_H)
 			{
-				if (uuid_parse(optarg, hint.uuid) < 0) {
+			  if (uuid_parse(optarg, (unsigned char *)hint.uuid) < 0) {
 					aal_error("Invalid uuid was "
 						  "specified (%s).",
 						  optarg);
@@ -305,19 +305,6 @@ int main(int argc, char *argv[]) {
 		goto error_free_libreiser4;
 	}
 
-	if (!(flags & BF_FORCE)) {
-		if (aal_strncmp(sysinfo.release, "2.5", 3) &&
-		    aal_strncmp(sysinfo.release, "2.6", 3) &&
-		    (aal_strncmp(sysinfo.release, "3.", 2) ||
-		     (sysinfo.release[2] < '0' && (sysinfo.release[2] > '9')))){
-			aal_warn("%s %s is detected. Reiser4 does not "
-				 "support such a platform. Use -f to "
-				 "force over.", sysinfo.sysname,
-				 sysinfo.release);
-			goto error_free_libreiser4;
-		}
-	}
-
 	if (!(flags & BF_YES)) {
 		aal_mess("%s %s is detected.", sysinfo.sysname,
 			 sysinfo.release);
@@ -406,14 +393,14 @@ int main(int argc, char *argv[]) {
 
 		/* Generating uuid if it was not specified and if libuuid is in use */
 #if defined(HAVE_LIBUUID) && defined(HAVE_UUID_UUID_H)
-		if (uuid_is_null(hint.uuid)) {
-			uuid_generate(hint.uuid);
+		if (uuid_is_null((unsigned char *)hint.uuid)) {
+			uuid_generate((unsigned char *)hint.uuid);
 		}
 
 		if (!(flags & BF_YES)) {
 			char uuid[256];
-				
-			uuid_unparse(hint.uuid, uuid);
+
+			uuid_unparse((unsigned char *)hint.uuid, uuid);
 			aal_mess("Uuid %s will be used.", uuid);
 		}
 #endif
